@@ -121,6 +121,13 @@ impl<'de> Parser<'de> {
         })
     }
 
+    pub(crate) fn parse_bytes<'s>(
+        &'s mut self,
+        scratch: &'s mut Vec<u8>,
+    ) -> Result<Reference<'de, 's, [u8]>> {
+        self.parse_str_bytes(scratch, |_, bytes| Ok(bytes))
+    }
+
     #[cold]
     #[inline(never)]
     fn parse_decimal_overflow(
@@ -390,6 +397,19 @@ impl<'de> Parser<'de> {
         })
     }
 
+    pub(crate) fn parse_ignore(&mut self) -> Result<()> {
+        while let Some(b) = self.next()? {
+            match b {
+                b'&' | b';' => {
+                    self.discard();
+                    break;
+                }
+                _ => {}
+            }
+        }
+        Ok(())
+    }
+
     pub(crate) fn parse_ident(&mut self, ident: &[u8]) -> Result<()> {
         for expected in ident {
             match self.next()? {
@@ -503,7 +523,7 @@ impl<'de> Parser<'de> {
         let mut key_index = self.index;
         while key_index < self.slice.len() {
             match self.slice[key_index] {
-                b'=' => {
+                b'=' | b'&' | b';' => {
                     key_found = true;
                     break;
                 }
@@ -544,7 +564,7 @@ impl<'de> Parser<'de> {
         }
 
         let slice = &self.slice[start_index..self.index];
-        return Ok(Some(slice));
+        Ok(Some(slice))
     }
 
     pub(crate) fn done(&self) -> bool {
